package be.cylab.java.wowa.training;

import info.debatty.java.aggregation.WOWA;

import java.util.List;

/**
 * Class of Solution. Solution with distance performance evaluation criterion
 */
public class SolutionDistance extends AbstractSolution {


    /**
     * SolutionDistance constructor. Needs weight number as parameter
     *
     * @param weights_number
     */
    public SolutionDistance(final int weights_number) {
        super(weights_number);
    }

    /**
     * Constructor used for tests.
     * @param weights_number
     * @param seed
     */
    public SolutionDistance(final int weights_number, final int seed) {
        super(weights_number, seed);
    }



    /**
     * Compute the solution fitness score.
     * For each element in the training dataset, compute WOWA and perform
     * a substraction between the WOWA just computed and the expected result
     * from the dataset.
     * The sum of all these differences is the fitness score (distance)
     * @param data
     * @param expected
     */
    final void computeScoreTo(final List<double[]> data,
                              final double[] expected
    ) {

        this.fitness_score = 0;
        for (int i = 0; i < data.size(); i++) {
            double[] vector = data.get(i);
            double target_value = expected[i];
            WOWA wowa = new WOWA(this.weights_w, this.weights_p);
            double aggregated_value = wowa.aggregate(vector);
            this.fitness_score += Math.pow(target_value - aggregated_value, 2);
        }
        this.fitness_score = Math.sqrt(this.fitness_score);

    }

}
